include "Vram.inc"
include "VVram.inc"
include "Chars.inc"

ext MonoPattern, ColorPattern, SpritePattern
ext VVram_

CharPatternSize equ VramStep*CharHeight
SpritePatternSize equ 4*2*16
Top equ VramRowSize*0

dseg
Backup:
	defs VVramWidth*VVramHeight
BackVram:
    defs VVramWidth*VVramHeight*VramStep*CharHeight
BackFlags:
    defs VVramHeight
CharRam: public CharRam
    defs CharPatternSize*Char_End

cseg
ColorSource:
defb 64,3 ; Ascii
defb 25,1 ; Logo
defb 6,3 ; Ladder
defb 0
InitVram: public InitVram
    mov ax,0004h | int 10h ; Set video mode
    mov ax,0500h | int 10h ; Select active display page

    mov si,MonoPattern
    mov di,CharRam
    mov bx,ColorSource
    do
        mov ch,[bx]
        or ch,ch
    while nz
        inc bx
        mov dl,[bx] | inc bx
        call MakePatternMono
    wend

                                        _deb: public _deb
    mov si,ColorPattern
    mov ch,Char_End-Char_Remain
    call MakePatternColor
ret

MakePatternMono:
;   di: destination
;   si: source
;   dl: color
;   ch: count
    mov dh,dl
    shr dh,1 | shr dh,1 | shr dh,1 | shr dh,1
    do
        mov cl,CharHeight
        do
            push cx
                mov al,[si] | inc si
                mov ch,VramStep
                do
                    xor ah,ah
                    mov cl,8/VramStep
                    do
                        shl ah,1 | shl ah,1
                        shl al,1
                        if c
                            or ah,dl
                        else
                            or ah,dh
                        endif
                        dec cl
                    while nz | wend
                    mov [di],ah | inc di
                    dec ch
                while nz | wend
            pop cx
            dec cl
        while nz | wend
        dec ch
    while nz | wend
ret
MakePatternColor:
;   di: destination
;   si: source
;   ch: count
    do
        mov cl,CharPatternSize
        do
            mov al,[si] | inc si
            mov [di],al | inc di
            dec cl
        while nz | wend
        dec ch
    while nz | wend
ret


; void ClearScreen();
cseg
Clear:
    xor ax,ax 
    do
        mov [bx],ax
        inc bx | inc bx
    wloop
ret
ClearScreen_: public ClearScreen_
    push ax | push bx | push cx
        mov ax,VramSegment | mov es,ax
        xor ax,ax
        mov bx,0
        do
            mov [es:bx],ax
            inc bx | inc bx
            cmp bx,4000h
        while nz | wend
        
		mov bx,Backup
		mov cx,VVramWidth*VVramHeight/2
        call Clear

		mov bx,BackVram
		mov cx,VVramWidth*VVramHeight*VramStep*CharHeight/2
        call Clear

		mov bx,BackFlags
		mov cx,VVramWidth/2
        call Clear
    pop cx | pop bx | pop ax
ret


; word Put(word vram, byte c);
cseg
Put_: public Put_
	push ax | push cx | push si | push di
	    mov di,ax
        mov ax,VramSegment | mov es,ax
        mov al,CharPatternSize | mul dl
        add ax,CharRam
        mov si,ax
        mov cl,CharHeight/2
        do
            mov ax,[si]
            inc si | inc si
            mov [es:di],ax
            mov ax,[si]
            inc si | inc si
            mov [es:di+2000h],ax
            add di,VramWidth
            dec cl
        while nz | wend
	pop di | pop si | pop cx | pop ax
	add ax,VramStep
ret


; void VVramToVram();
cseg
VVramToVram_: public VVramToVram_
	push ax | push cx | push dx | push bx | push si | push di
        mov dx,BackFlags
        mov bx,VVram_
		mov si,Backup
		mov di,BackVram
		mov ch,VVramHeight ; vertical count
		do
			mov cl,VVramWidth
			do 
				mov al,[bx] | inc bx
				cmp al,[si]
				if ne
					mov [si],al
                    push cx | push si
                        mov ah,CharPatternSize | mul ah
                        add ax,CharRam
                        mov si,ax
                        mov cl,CharHeight
                        do
                            mov ax,[si]
                            inc si | inc si
                            mov [di],ax
                            add di,VVramWidth*VramStep
                            dec cl
                        while nz | wend
                    pop si | pop cx
                    sub di,VVramWidth*VramStep*CharHeight

                    xchg dx,bx
                    mov byte ptr[bx],0ffh
                    xchg dx,bx
				endif
				inc di | inc di
				inc si
				dec cl
			while nz | wend
			add di,VVramWidth*VramStep*CharHeight-VVramWidth*VramStep
            inc dx
			dec ch
		while nz | wend
	pop di | pop si | pop bx | pop dx | pop cx | pop ax
ret


; void PresentVram();
cseg
PresentVram_: public PresentVram_
    push ax | push cx | push dx | push bx | push si | push di
        mov ax,VramSegment | mov es,ax
        mov si,BackVram
        mov di,Vram+Top
        mov bx,BackFlags
        mov dh,03h
        mov dl,[bx] | mov byte ptr[bx],0 | inc bx
        mov ch,VVramHeight*CharHeight/2
        do
            test dh,dl
            if nz
                mov cl,VVramWidth
                do
                    mov ax,[si]
                    cmp ax,[es:di]
                    if nz
                        mov [es:di],ax
                    endif
                    inc si | inc si
                    inc di | inc di
                    dec cl
                while nz | wend
                add di,2000h-VVramWidth*VramStep
                mov cl,VVramWidth
                do
                    mov ax,[si]
                    cmp ax,[es:di]
                    if nz
                        mov [es:di],ax
                    endif
                    inc si | inc si
                    inc di | inc di
                    dec cl
                while nz | wend
                add di,-2000h-VVramWidth*VramStep
            else
                add si,VVramWidth*VramStep*2
            endif
            add di,VramWidth
            shl dh,1 | shl dh,1
            if z
                mov dh,03h
                mov dl,[bx] | mov byte ptr[bx],0 | inc bx
            endif
            dec ch
        while nz | wend
    pop di | pop si | pop bx | pop dx | pop cx | pop ax
ret


; void EraseBackup(byte x, byte y);
cseg
EraseBackup_: public EraseBackup_
	push ax | push dx | push cx | push bx
		mov cx,0202h
		test al,3
		if nz
			inc ch
		endif
		shr al,1 | shr al,1
		mov bl,al
		xor bh,bh
		
		test dl,3
		if nz
			inc cl
		endif
		shr dl,1 | shr dl,1

		mov al,VVramWidth
		mul dl
		add ax,bx
		add ax,Backup
		mov bx,ax

		mov al,0ffh
		do
			push bx
				mov ah,ch
				do
					mov [bx],al
					inc bx
					dec ah
				while nz | wend
			pop bx
			add bx,VVramWidth
			dec cl
		while nz | wend
 	pop bx | pop cx | pop dx | pop ax
ret


; void DrawSprite(byte x, byte y, byte pattern);
cseg
DrawSprite_: public DrawSprite_
    push ax | push bx | push cx | push dx | push si | push di
        mov ch,cl ; pattern
        mov cl,al ; x

        mov al,dl
        shr al,1 | shr al,1
        xor ah,ah
        add ax,BackFlags
        mov bx,ax
        mov ax,0ffffh
        mov [bx],ax | inc bx
        mov [bx],ax

        mov ah,dl ; y
        xor al,al
        shr ax,1
        add ax,BackVram
        mov dl,cl ; x
		shr dl,1
        xor dh,dh
        add ax,dx
        mov di,ax

		and cl,3
		shl cl,1

		mov ah,ch ; pattern
		xor al,al
        shr ax,1
		add ax,SpritePattern
		mov si,ax

        mov dh,16
        do
            mov dl,4
            do
                push dx
                    mov dh,[si] | inc si
                    xor dl,dl
                    shr dx,cl
                    xchg dh,dl
                    not dx
                    mov ax,[di]
                    and ax,dx
                    mov dh,[si] | inc si
                    xor dl,dl
                    shr dx,cl
                    xchg dh,dl
                    or ax,dx
                    mov [di],ax
					inc di
                pop dx
                dec dl
            while nz | wend
			add di,VVramWidth*VramStep-4
            dec dh
        while nz | wend
    pop di | pop si | pop dx | pop cx | pop bx | pop ax
ret
